@@ -9,7 +9,7 @@ use Schedule::Activity::Message;
 use Schedule::Activity::Node;
 use Schedule::Activity::NodeFilter;
 
-our $VERSION='0.1.7';
+our $VERSION='0.1.8';
 
 sub new {
 	my ($ref,%opt)=@_;
@@ -20,6 +20,7 @@ sub new {
 		valid   =>0,
 		built   =>undef,
 		reach   =>undef,
+		unsafe  =>$opt{unsafe}//1, # default to 1 initially
 	);
 	return bless(\%self,$class);
 }
@@ -78,6 +79,7 @@ sub _reachability { # UNTESTED (directly)
 		foreach my $nodea (keys %{$reach{max}}) {
 		foreach my $nodeb (keys %{$reach{max}{$nodea}}) {
 		foreach my $nodec (keys %{$reach{max}{$nodeb}}) {
+			if($nodea eq $nodec) { $reach{max}{$nodea}{$nodec}='+'; next }
 			my $x=$reach{max}{$nodea}{$nodec};
 			if(defined($x)&&($x eq '+')) { next }
 			my $y=&$triadd($reach{max}{$nodea}{$nodeb},$reach{max}{$nodeb}{$nodec});
@@ -121,8 +123,8 @@ sub safetyChecks {
 	foreach my $action (keys %actions) {
 		my $parents=0;
 		my $terminals=0;
-		foreach my $activity (keys %activities) { if($reach{min}{$activity}{$actions{$action}}) { $parents++ } }
-		foreach my $finish   (keys %finishes)   { if($reach{min}{$actions{$action}}{$finish})   { $terminals++ } }
+		foreach my $activity (keys %activities) { if(defined($reach{min}{$activity}{$actions{$action}})) { $parents++ } }
+		foreach my $finish   (keys %finishes)   { if(defined($reach{min}{$actions{$action}}{$finish}))   { $terminals++ } }
 		if($parents==0)    { $orphans{$action}=1 }
 		elsif($parents>1)  { $dualParent{$action}=1 }
 		if($terminals>1)   { $dualFinish{$action}=1 }
@@ -138,24 +140,24 @@ sub safetyChecks {
 }
 
 sub compile {
-	my ($self)=@_;
+	my ($self,%opt)=@_;
 	if($$self{built}) { return }
 	my @errors=$self->validate();
 	if(@errors) { return (error=>\@errors) }
 	%{$$self{built}}=buildConfig(%{$$self{config}});
-	# @errors=$self->safetyChecks();
-	# if(@errors) { return (error=>\@errors) }
+	if(!$opt{unsafe}) { @errors=$self->safetyChecks(); if(@errors) { return (error=>\@errors) } }
 	return;
 }
 
 sub schedule {
 	my ($self,%opt)=@_;
 	delete($$self{attr});
-	my %check=$self->compile(); if($check{error}) { return (error=>$check{error}) }
-	if(!is_arrayref($opt{activities}))            { return (error=>'Activities must be an array') }
+	my %check=$self->compile(unsafe=>$opt{unsafe}//$$self{unsafe});
+	if($check{error})                  { return (error=>$check{error}) }
+	if(!is_arrayref($opt{activities})) { return (error=>'Activities must be an array') }
 	my ($tmoffset,%res)=(0);
 	foreach my $activity (@{$opt{activities}}) {
-		foreach my $entry (scheduler(goal=>$$activity[0],node=>$$self{built}{node}{$$activity[1]},config=>$$self{built},attr=>$self->_attr(),tmoffset=>$tmoffset)) {
+		foreach my $entry (scheduler(goal=>$$activity[0],node=>$$self{built}{node}{$$activity[1]},config=>$$self{built},attr=>$self->_attr(),tmoffset=>$tmoffset,tensionslack=>$opt{tensionslack},tensionbuffer=>$opt{tensionbuffer})) {
 			push @{$res{activities}},[$$entry[0]+$tmoffset,@$entry[1..$#$entry]];
 		}
 		$tmoffset+=$$activity[0];
@@ -430,7 +432,7 @@ Schedule::Activity - Generate random activity schedules
 
 =head1 VERSION
 
-Version 0.1.7
+Version 0.1.8
 
 =head1 SYNOPSIS
 
@@ -529,7 +531,7 @@ The slack is the amount of time that could be reduced in an action before it wou
 
 Providing any time value will automatically set any missing values at the fixed ratios 3,4,5.  EG, specifying only C<tmmax=40> will set C<tmmin=24> and C<tmavg=32>.  If provided two time values, priority is given to C<tmavg> to set the third.
 
-Future changes may support adjusting these ratios, automatic slack/buffering, univeral slack/buffer ratios, and open-ended/relaxed slack/buffering.
+Scheduling may be controlled with the tension settings described below.  Future changes may support automatic slack/buffering, univeral slack/buffer ratios, and open-ended/relaxed slack/buffering.
 
 =head2 Messages
 
@@ -587,6 +589,24 @@ The difference between the result time and the goal may cause retries when an ex
 
 Caution:  While startup/conclusion of activities may have fixed time specifications, at this time it is recommended that actions always contain some slack/buffer.  There is currently no "relaxing mechanism" during scheduling, so a configured with no slack nor buffer must exactly meet the goal time requested.
 
+=head1 SCHEDULING TENSION
+
+Schedule construction proceeds toward the goal time incrementally, with each action appending its C<tmavg> until the goal is reached.  If the accumulated average times were exactly equal to the goal for the activity, schedules would be unambiguous.  For repeating, recursive scheduling, however, it's necessary to consider scenarios where the actions don't quite reach the goal or where they extend beyond the goal.
+
+=head2 Buffer
+
+The primary method to handle differences between the scheduled time and goal time is the C<buffer>.  As defined above, each activity with C<tmmaxE<gt>tmavg> contributes to the total buffer in the schedule.  The amount of buffer used to select the next activity is controlled by including C<schedule(tensionbuffer=>value)>.  In the 'laziest' mode, C<tensionbuffer=0.0>, all available buffer contributes toward achieving the goal, meaning that scheduling will attempt to reach the final activity node sooner, and schedules will effectively contain a smaller number of activities, each stretched toward C<tmmax>.  In the most aggressive mode, C<tensionbuffer=1.0>, the goal time must be met (or exceeded) before aggressively seeking the final activity node, so schedules will contain a larger number of activities, each compressed toward C<tmmin>.  The default is 0.5.
+
+=head2 Slack
+
+The second method to handle differences is the C<slack>, which can be controlled with C<schedule(tensionslack=>value)>.  With C<tensionslack=0.0>, all accumulated slack will be used to schedule activities beyond the goal time, so schedules will effectively contain a larger number of activities compressed toward C<tmmin>.  With C<tensionslack=1.0>, scheduling will seek the final activity node as soon as the schedule time exceeds the goal, resulting in a smaller number of activities.  The default is 0.5.
+
+Note that the slack tension is secondary to the buffer tension.  With the default values, on average, it's more likely that the buffer will be used to reach the goal.  That is, the number of actions is not uniformly distributed around the C<goal/tmavg> count, but biased toward the lower side.
+
+=head2 Example
+
+Suppose a single repeatable action with C<tmavg=10> is used to construct a schedule with C<goal=99>.  The expected action count is 10 when C<tensionbuffer=1.0> and C<tensionslack=1.0>.  If C<tensionbuffer> is closer to 0.0, the expected number of actions is smaller than 10, perhaps even as low as 1 based on C<tmmax>, because the scheduler determines that more of the buffer can be used to stretch the actions toward the goal of 99.  If, on the other hand, C<tensionslack> is closer to 0.0, the expected number of actions is greater than 10, perhaps even as high as 19 based on C<tmmin>, because the scheduler attempts to insert more non-final actions that can be compressed toward the goal.
+
 =head1 ATTRIBUTES
 
 Attributes permit tracking boolean or numeric values during schedule construction.  The result of C<schedule> contains attribute information that can be used to verify or adjust the schedule.
@@ -670,7 +690,7 @@ The reported C<xy> is an array of values of the form C<(tm, value)>, with each r
 
 For integers, attributes may be fixed in the log at their current value by calling C<incr=0>.  There is currently no similar mechanism for booleans.
 
-(Approaching a decision):  As of version 0.1.1, attribute logging will also occur at the end of every activity, so changes in attributes across activity boundaries do not affect the average value calculation.  In particular, the starting value in any given activity is the most recent value in the previous activity, adjusted by any operator in the activity node itself.  For example, suppose two activities go from C<tm=0> to 10, and from C<tm=10> to 20.  If an attribute is set to C<tm=0, value=5> and not set again until C<tm=15, value=0>, then the average in the first activity is five.
+(Approaching a decision):  Attribute logging will also occur at the end of every activity, so changes in attributes across activity boundaries do not affect the average value calculation.  In particular, the starting value in any given activity is the most recent value in the previous activity, adjusted by any operator in the activity node itself.  For example, suppose two activities go from C<tm=0> to 10, and from C<tm=10> to 20.  If an attribute is set to C<tm=0, value=5> and not set again until C<tm=15, value=0>, then the average in the first activity is five.
 
 Proposed:  Because C<incr=0> can fix the value of an integer attribute in the final action node of an activity, this permits the user to choose the behavior.  For Boolean attributes, they are already fixed until the next C<set> event, so the average value should be equivalent whether these are pinned at the end of the activity or not.
 
